package cn.iocoder.yudao.module.system.service.attendance;

import cn.hutool.http.HttpUtil;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import cn.iocoder.yudao.framework.common.util.date.DateUtils;
import cn.iocoder.yudao.module.system.controller.app.attendance.dto.AttendancePunchPageDTO;
import cn.iocoder.yudao.module.system.controller.app.attendance.vo.AttendancePunchPageVO;
import cn.iocoder.yudao.module.system.controller.app.attendance.vo.HolidayVO;
import cn.iocoder.yudao.module.system.dal.dataobject.attendance.group.AttendanceGroupDO;
import cn.iocoder.yudao.module.system.handler.PunchHandler;
import cn.iocoder.yudao.module.system.service.attendance.group.AttendanceGroupService;
import cn.iocoder.yudao.module.system.service.attendance.punch.PunchService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Date;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 考勤 实现类
 */
@Service
@Slf4j
public class AttendanceServiceImpl implements AttendanceService {
    @Resource
    private PunchHandler punchHandler;
    @Resource
    private AttendanceGroupService attendanceGroupService;
    @Resource
    private StringRedisTemplate stringRedisTemplate;

    @Override
    public AttendancePunchPageVO getPunchPage(AttendancePunchPageDTO dto) {
        AttendancePunchPageVO vo = new AttendancePunchPageVO();
        //获取当前登录用户所在群组
        AttendanceGroupDO activationGroup = attendanceGroupService.getByUserId(dto.getUserId());
        //不在考勤组 - 返回回去
        if (activationGroup == null) {
            vo.setInGroup(AttendanceGroupDO.FALSE);
            return vo;
        }
        // 策略模式 - 将不同考勤类型的 分散开逻辑 - 后续好拓展
        PunchService punchService = punchHandler.getResource(AttendanceGroupDO.getCodeByType(activationGroup.getType()));
        return punchService.getPunchPage(dto.setActivationGroup(activationGroup));
    }

    @Override
    public Boolean isHoliday(Date date) {
        String dateStr = DateUtils.dateFormat(date, "yyyy-MM-dd");
        String year = dateStr.substring(0, 4);
        String key = "holiday_" + year;
        Boolean flag = stringRedisTemplate.hasKey(key);
        // 缓存不存在
        if (Boolean.FALSE.equals(flag)) {
            String url = "https://cdn.jsdelivr.net/gh/NateScarlet/holiday-cn@master/" + year + ".json";
            String json = HttpUtil.get(url, 0);
            JSONObject jsonObject = JSONUtil.parseObj(json);
            Map<String, Boolean> map = jsonObject.getBeanList("days", HolidayVO.class).stream().collect(Collectors.toMap(HolidayVO::getDate, HolidayVO::getIsOffDay));
            stringRedisTemplate.opsForHash().putAll(key, map);
            // -- 删除去年的
            String lastKey = "holiday_" + (Integer.parseInt(year) - 1);
            stringRedisTemplate.delete(lastKey);
        }
        return (Boolean) stringRedisTemplate.opsForHash().get(key, date);
    }

}
