package cn.iocoder.yudao.module.system.service.attendance.scheduling;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.module.system.controller.admin.scheduling.vo.AttendanceSchedulingPageReqVO;
import cn.iocoder.yudao.module.system.controller.admin.scheduling.vo.AttendanceSchedulingSaveReqVO;
import cn.iocoder.yudao.module.system.controller.app.attendance.dto.AttendancePunchPageDTO;
import cn.iocoder.yudao.module.system.controller.app.attendance.vo.AttendancePunchPageVO;
import cn.iocoder.yudao.module.system.dal.dataobject.attendance.scheduling.AttendanceSchedulingDO;
import cn.iocoder.yudao.module.system.dal.mysql.attendance.scheduling.AttendanceSchedulingMapper;
import cn.iocoder.yudao.module.system.service.attendance.punch.PunchService;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.system.enums.ErrorCodeConstants.SCHEDULING_NOT_EXISTS;

/**
 * 排班制考勤设置 Service 实现类
 *
 * @author 艾楷
 */
@Service("scheduling")
@Validated
public class AttendanceSchedulingServiceImpl implements AttendanceSchedulingService, PunchService {

    @Resource
    private AttendanceSchedulingMapper schedulingMapper;

    @Override
    public Long createScheduling(AttendanceSchedulingSaveReqVO createReqVO) {
        // 插入
        AttendanceSchedulingDO scheduling = BeanUtils.toBean(createReqVO, AttendanceSchedulingDO.class);
        schedulingMapper.insert(scheduling);
        // 返回
        return scheduling.getId();
    }

    @Override
    public void updateScheduling(AttendanceSchedulingSaveReqVO updateReqVO) {
        // 校验存在
        validateSchedulingExists(updateReqVO.getId());
        // 更新
        AttendanceSchedulingDO updateObj = BeanUtils.toBean(updateReqVO, AttendanceSchedulingDO.class);
        schedulingMapper.updateById(updateObj);
    }

    @Override
    public void deleteScheduling(Long id) {
        // 校验存在
        validateSchedulingExists(id);
        // 删除
        schedulingMapper.deleteById(id);
    }

    private void validateSchedulingExists(Long id) {
        if (schedulingMapper.selectById(id) == null) {
            throw exception(SCHEDULING_NOT_EXISTS);
        }
    }

    @Override
    public AttendanceSchedulingDO getScheduling(Long id) {
        return schedulingMapper.selectById(id);
    }

    @Override
    public PageResult<AttendanceSchedulingDO> getSchedulingPage(AttendanceSchedulingPageReqVO pageReqVO) {
        return schedulingMapper.selectPage(pageReqVO);
    }

    @Override
    public AttendancePunchPageVO getPunchPage(AttendancePunchPageDTO dto) {
        return null;
    }
}